package com.laihua.projecthelper.implement

import com.laihua.projecthelper.ParseUtil
import com.laihua.projecthelper.ReadUtil
import java.io.BufferedReader
import java.io.File
import java.io.InputStreamReader

/**
 * Author: pengjunjing
 * Date: 2022/3/7
 * Description:单文件引用工具类,用于抽取到对应模块的时候使用
 */


/**
 * 是否真的会执行移动操作的变量,请对当前git的操作进行贮存,
 * 确保项目没有其他操作后再执行,否则可能会操作记录混乱
 */
const val realExecuteCmd = false

fun main(args: Array<String>) {
    println("开始读取文件")
    val projectRoot = File("")
    val projectPath = projectRoot.absolutePath
    val projectRootF = File(projectPath)
    //这里打印的应该会是as项目所处的路径
    println(projectPath)
    //要检查资源的目录,大小写敏感
    val baseFile = File("$projectPath${File.separator}LaiHuaBase${File.separator}src${File.separator}")
    //从项目根目录开始查找
//    val baseFile = File("${projectRoot.absolutePath}")
    println(baseFile.absolutePath + "\n")
    val readImageRes = ReadUtil().readImageRes(baseFile)
    val readCodeFile = ReadUtil().readCodeFile(projectRootF)


    //查找单个依赖的文件
    val singleList: List<FileCalc> = startFind(readImageRes, readCodeFile)
    //这个单模块依赖的意思,包含本模块依赖的文件
    println("一共有:${singleList.size}个文件是单模块依赖的,都需要进行移动")
    val shellCmd = getShellCmd(singleList, projectPath)
    println("------------------------------")
    /**
     * 当文件很多时,该遍历会导致一行一行执行命令,窗口一闪一闪的,效率比较低.
     * 如果追求效率可以使用写入文件,然后使用bat或者shell的方式一次执行
     */
    shellCmd.forEach { cmd ->
        println("将要执行命令:$cmd")
        if (realExecuteCmd) {
            exeCmd(cmd)
        }
    }
    //写入到文件
//    writeToFile(shellCmd, projectPath)


}


/**
 * 获取到命令行
 * 如果执行的命令结果提示:
 * fatal: not under version control, source=m_kt_base/README.MD, destination=m_kt_account/README.MD
 * 请检查文件夹大小是否一样
 */
private fun getShellCmd(singleList: List<FileCalc>, projectPath: String): List<String> {
    val cmdList = mutableListOf<String>()
    for (fileCalc in singleList) {
        val implementModuleName = fileCalc.implementModule.toList()[0]
        if (fileCalc.belongModule != null && implementModuleName.lowercase() != fileCalc.belongModule.lowercase()) {
            println("模块:$implementModuleName,依赖了 ${fileCalc.belongModule} 模块下的文件: ${fileCalc.file.name}")
            val srcPath = fileCalc.file.absolutePath
            val targetPath =
                srcPath.replace(fileCalc.belongModule!!, fileCalc.implementModule.toList()[0])
            val targetFile = File(targetPath)
            println("需要将:$srcPath 移动到: $targetPath")
            if (realExecuteCmd && !targetFile.parentFile.exists()) {
                //在确认需要执行命令的情况下 , 文件夹不存在则创建文件夹
                val mkdirs = targetFile.parentFile.mkdirs()
                println("创建文件夹:${targetFile.parentFile.absolutePath}  $mkdirs")
            }
            if (targetFile.parentFile.exists()) {
                //执行命令行
                val form = srcPath.replace(projectPath, "")
                val to = targetPath.replace(projectPath, "")
                val cmd = "git mv .$form .$to"
                cmdList.add(cmd)
            }
        }
    }
    return cmdList
}

/**
 * 执行cmd命令
 */
private fun exeCmd(cmd: String) {
    println(cmd)
    val cmdHead = "cmd.exe /C start "
    val process = Runtime.getRuntime().exec(cmdHead + cmd)
    val status = process.waitFor()
    val bufferedReader = BufferedReader(InputStreamReader(process.inputStream))
    var readLine = bufferedReader.readLine()
    while (readLine != null) {
        println(readLine)
        readLine = bufferedReader.readLine()
    }
    println("------------------")
}

/**
 * 从资源文件中,返回只被一个模块依赖的文件.
 */
fun startFind(readImageRes: List<File>, readCodeFile: List<File>): List<FileCalc> {
    //统计所有有调用到资源的代码文件
    val codeFileList = mutableListOf<CodeFileFindResult>()
    readCodeFile.forEach {
        val parseFile = ParseUtil().parseFile(it)
        if (parseFile.isNotEmpty()) {
            codeFileList.add(CodeFileFindResult(it, parseFile))
        }

    }

    val resList: List<FileCalc> = readImageRes.map {
        FileCalc(it)
    }

    //统计引用次数
    codeFileList.forEach { codeFile: CodeFileFindResult ->//代码文件
        codeFile.codeLine.forEach { codeLine: String ->//一行代码
            resList.forEach { resFile: FileCalc ->//资源文件
                if (codeLine.contains(resFile.implementName, ignoreCase = true)) {
                    //引用次数+1
                    resFile.implementCode.add(codeFile)
                }
            }
        }
    }

    resList.forEach { fileCalc ->
        //计算所有依赖是属于哪个模块的
        fileCalc.calcImplementModule()
    }

    //只有一个依赖的列表
    val singleList = mutableListOf<FileCalc>()
    resList.forEach { fileCalc ->

        if (fileCalc.implementModule.size == 1) {
            //只有一个模块依赖的该文件,移动到
            singleList.add(fileCalc)
        }
    }

    //按模块排序
    singleList.sortBy {
        it.implementModule.toList()[0]
    }

    return singleList
}


fun writeToFile(shellCmd: List<String>, projectPath: String) {
    val batFile = File("$projectPath${File.separator}singleImplement.bat")
//    batFile.deleteOnExit()
    val createNewFile = batFile.createNewFile()
    println("创建文件:${batFile.absolutePath} $createNewFile")
    //清空内容
    batFile.writeText("")
    shellCmd.forEach {
        batFile.appendText(it + "\n")
    }
    batFile.appendText("pause")
}
